#############################################################################
# Diese Datei enthlt Routinen zum Lesen und Schreiben von Benutzerinfos
# Erstellt von Markus Zhlke, 1998-06
# Dieser Quellcode darf durch den Arbeitskreis Linux am IFL Hamburg
# beliebig veraendert werden.
# Die Kopierrechte liegen bei dem Arbeitskreis Linux am IFL Hamburg.
# Es wird keine Haftung fuer entstehende Schaeden genommen.
#       Markus Zuehlke, Hamburg, 1998
#

#############################################################################
# Formate der Dateien ...
# $HTPASSFILE	Ist eine Passwortdatei fr den WWW-Server
# -rw-------	Die Datei sollte dem WWW-Server gehren
#		Loginname:Passwort
# 
# shadow	Shadow-Passwortdatei
# -rw-------	Loginname:Passwort:0:0:99999:7:::0
# 		Die Angaben der shadow haben spezielle Bedeutungen, werden
#		hier aber auf "Null"-Werte gesetzt.
#
# passwd	Benutzer-Datei
# -rw-r--r--	Loginname:x:uid:gid:Name:Home:Shell
#
# group		Gruppen-Datei
# -rw-r--r--	Gruppenname:x:gid:Benutzer der Gruppe
#

#############################################################################
# Routinen die in dieser Datei enthalten sind ...
#
# getuserdata  	Ermittelt die Benutzerdaten zu einem Loginname
# 		Parameter1 : Loginname
#		Rckgabe   : Array mit Loginname, Passwort(E), Name, UID, 
#			     GID, Home, Shell, Recht, Klasse, KlassenID, 
#			     Lehrer, LehrerID
#
# Recht : Wert 0=Other, 1, Admin, 2=Lehrer, 3=Schler
# Genaueres siehe config.pl
# ------------------------------------------------------------------------
# getgroups	Gibt eine Liste mit allen Gruppen zurueck, in denen der
#		Benutzer Mitglied ist.
# 		Parameter1 : Loginname
#		Rueckgabe  : Liste mit den Gruppen zurueck.
# ------------------------------------------------------------------------
# checkgroup	Testet ob ein Benutzer in einer Gruppe Mitglied ist.
#		Parameter1 : Loginname
#		Parameter2 : Gruppenname
#		Rueckgabe  : 1:Benutzer ist Mitglied, 0:Benutzer ist nicht
#			     Mitglied.
# ------------------------------------------------------------------------
# getuserlist	Gibt eine Liste mit allen Benutzer zurueck, deren UID im
#		Bereich der Definition in der config.pl ist.
#		Rueckgabe  : Liste mit den Loginnamen
# ------------------------------------------------------------------------
# getgrouplist	Gibt eine Liste mit allen Gruppen zurueck, deren GID im
#		Bereich der Definition in der config.pl ist.
# ------------------------------------------------------------------------
# getuidbyuser	Ermittelt die UID eines Benutzers
# 		Parameter1 : Loginname
#		Rueckgabe  : UID des Benutzers oder -1
# ------------------------------------------------------------------------
# getallgroups	Packt alle Gruppen (config.pl) ausser der SA-Gruppen in 
#		ein Array.
#		Rueckgabe  : Liste mit den /etc/group-Eintraegen
# ------------------------------------------------------------------------
# getallallgroups Gibt eine Liste aller Gruppen (config.pl) ausser der
#		SA-Gruppen zurueck.
#		Rueckgabe  : Liste mit den Gruppennamen
# ------------------------------------------------------------------------
# getgroupbygid	Ermittelt den Gruppenname zu einer GID.
#		Parameter1 : GID
#		Rueckgabe  : Name der Gruppe
# ------------------------------------------------------------------------
# getgidbygroup Ermittelt die GID zu einer Gruppe.
#		Parameter1 : Gruppenname
#		Rueckgabe  : -1 oder GID
# ------------------------------------------------------------------------
# html_userlist	Gibt einen String in HTML-Form fuer Listen zurueck, in
#		denen alle Benutzer angegeben sind. (config.pl)
#		Rueckgabe  : String in folgender Form:
#		<OPTION>[Name]<OPTION>[Name](...)
# ------------------------------------------------------------------------
# html_groups	Gibt alle Gruppen in HTML-Form fuer Listen zurueck. Alle
#		Gruppen, in denen der Benutzer Mitglied(sekundaer) ist, 
#		werden mit SELECTED zurueckgegeben.
#		Parameter1 : Loginname
#		Rueckgabe  : String im HTML-Listen-Format(s. o.)
# ------------------------------------------------------------------------
# html_shells	Gibt eine Liste aller Shells aus /etc/shells zurueck.
#		Es wird ein String in HTML-Listen-Form zurueckgegeben, die
#		Shell, die den Parameter angibt wird mit SELECTED markiert
#		Parameter1 : Shell+Pfad
#		Rueckgabe  : String in HTML-Listen-Format(s. o.)
# ------------------------------------------------------------------------
# html_saright	Gibt einen String in HTML-Listen-Form zurueck der die 
# 		SA-Gruppen enthaelt.
#		Parameter1 : 0-3 (Gruppe die SELECTED sein soll)
#		Parameter2 : 0-3 (Recht der Person, die aendern will.
#		Rueckgabe  : String in HTML-Listen-Form(s. o.)
# ------------------------------------------------------------------------
# html_allgroups Gibt eine Liste aller Gruppen in HTML-Listen-Form aus.
#		Parameter1 : GID, der Gruppe die SELECTED sein soll.
# 		Rueckgabe  : String in HTML-Listen-Form(s. o.)
# ------------------------------------------------------------------------
# convert_special_chars Wandelt Sonderzeichen in Standart zeichen.
#		Parameter1 : String mit Sonderzeichen (z.B. AUmlaut)
#		Rueckgabe  : String ohne Sonderzeichen (zu ae)
# ------------------------------------------------------------------------
# createnewuser Erstellt einen Account. Achtung, die Funktion hat Ausgaben
# 		Parameter  : Liste mit 8 Feldern
# 		_[0] = VollerName _[1] = username, _[2] = passwort, 
#		_[3] = homedir, _[4] = SA-Recht, _[5] = Klasse, 
#		_[6] = Shell, _[7] = ExtraStr 
# _[1] = "", dann ist username = xx0000 xx=initialen 0000 = Zahl ab 1000
# _[2] = "", dann ist passwort = Vorname in klein
# _[3] = "", dann ist homedir = "/home/[username]"
# _[6] = "", dann ist shell = "Keine Shell"
# _[7] ist fuer Klasse, anlegender Lehrer da.
#		Rueckgabe  : Keine
# ------------------------------------------------------------------------
# getclassid 	Ermittelt eine ID fuer eine Klasse. Erstellt eine neue ID
#		Parameter1 : Name der Klasse (z.B. 10b)
#		Rueckgabe  : Zahl zwischen 1000 und groesser
# ------------------------------------------------------------------------
# getlname	Gibt den vollen Name eines Benutzers zurueck.
#		Parameter1 : Loginname
#		Rueckgabe  : voller Name
# ------------------------------------------------------------------------
# getuseringroup Gibt eine Liste mit allen Benutzern die in einer Gruppe
#		Mitglied sind zurueck.
# ------------------------------------------------------------------------
# createnewgroup Erstellt eine neue Gruppe. Achtung diese Funktion hat
#		Ausgaben.
# 		Parameter1 : grpname
# ------------------------------------------------------------------------
# getlehrer	Ermittelt den Lehrerloginnamen zu einem Loginname
#		Parameter1 : Loginname1
#		Rueckgabe  : Loginname2 des Lehrers der fuer Loginname1
#			     zustaendig ist.
# ------------------------------------------------------------------------
# getauth	Gibt true zurueck, wenn modinfo.dat zugang erlaubt.
#		Parameter1 : WebTools-Gruppe (0-3)
#		Rueckgabe  : true oder false
# ------------------------------------------------------------------------
# ------------------------------------------------------------------------
# 

#Gibt eine Liste aller Gruppen zurueck.
sub getgroups {
  my $username = $_[0];
  local($groupres, @groups);
  
  $groupres = `groups $username`;
  chomp($groupres);
  @groups = split(/:/,$groupres);
  @groups = split(/\ /,$groups[1]);
  shift(@groups);
#  shift(@groups);
  return @groups;
}

# Gibt 1 zurueck, wenn ein Benutzer in der Gruppe vorhanden ist.
# Parameter: username, gruppe
sub checkgroup {
  my $username = $_[0];
  my $gruppenname = $_[1];
  local($res, @gruppen);
  
  @gruppen = getgroups($username);
  $res = 0;
  foreach (@gruppen) {
    if ($gruppenname eq $_) { $res = 1; }
  }
  return $res;
}

sub getuserdata {
  my $username = $_[0];
  local(@shadowdata, @userinfodata, @passwddata);
  local($shadowstring, $userinfostring, $passwdstring);
  local(@shadowteile, @userinfoteile, @passwdteile); 
  local(@resultat, @GRPARRAY, $GRPSTRING, @tmp);
  local(@extuserdata);
  
  open(SHADOW, "</etc/shadow"); # shadow auslesen
  @shadowdata = <SHADOW>;
  close(SHADOW);
  open(PASSWD, "</etc/passwd"); # passwd auslesen
  @passwddata = <PASSWD>;
  close(PASSWD);
  foreach(@passwddata) {
    chomp;
    @passwdteile = split(/:/,$_,-1);
    if ($passwdteile[0] eq $username) {
      $passwdstring = $_;
      last;
    }
    undef(@passwdteile);
  }
  @tmp = split(/,/,$passwdteile[4],-1);
  $passwdteile[4] = $tmp[0];
  if (@tmp > 4) {
    @extuserdata = split(/;/,$tmp[4],-1);
  } else {
    @extuserdata = ("","","","");
  }
  # Klasse
  # KlassenID
  # Lehrer
  # LehrerID (username)
  foreach(@shadowdata) {
    chomp;
    @shadowteile = split(/:/,$_,-1);
    if ($shadowteile[0] eq $username) {
      $shadowstring = $_;
      last;
    }
    undef(@shadowteile);
  }
  if ($passwdstring eq "") {
    return ;
  }
  if ($shadowstring eq "") {
    return ;
  }
  $GRPARRAY[0] = 100;
  foreach (@SAGRUPPENNAMEN) {
    if (checkgroup($username, $_) == 1) {
      if ($GRUPPENRECHT{$_}<$GRPARRAY[0]) {
        $GRPARRAY[0] = $GRUPPENRECHT{$_};
      }
    }
  }
  @resultat = ($passwdteile[0],$shadowteile[1],$passwdteile[4],
  $passwdteile[2],$passwdteile[3],$passwdteile[5],$passwdteile[6],$GRPARRAY[0],
  $extuserdata[0],$extuserdata[1],$extuserdata[2],$extuserdata[3]);
  return @resultat;
}

# Gibt eine Liste mit allen Benutzernamen zurck.
# Wenn die UserID kleiner $FIRST_CH_UID ist wird der Eintrag nicht zurck
# gegeben.
sub getuserlist {
  local (@pwdlist, @unamelist, @dummy);
  
  open PASSWD, "</etc/passwd";
  @pwdlist = <PASSWD>;
  close PASSWD;
  foreach (@pwdlist) {
    chomp;
    @dummy = split(/:/,$_,-1);
    if ((@dummy == 7) && ($dummy[2]>=$FIRST_CH_UID) && ($dummy[2]<=$LAST_CH_UID)) {
      push @unamelist, $dummy[0];
    }
    undef(@dummy);
  }
  
  return @unamelist;
}

# Gibt eine Liste mit allen Gruppennamen zurck.
# Wenn die GID kleiner $FIRST_CH_GID ist wird der Eintrag nicht zurck
# gegeben.
sub getgrouplist {
  local (@grplist, @gnamelist, @dummy);
  
  open GROUP, "</etc/group";
  @grplist = <GROUP>;
  close GROUP;
  foreach (@grplist) {
    chomp;
    @dummy = split(/:/,$_,-1);
    if ((@dummy == 4) && ($dummy[2]>=$FIRST_CH_GID) && ($dummy[2]<=$LAST_CH_GID)) {
      push @gnamelist, $dummy[0];
    }
    undef(@dummy);
  }
  
  return @gnamelist;
}

# Ermittelt die UID eines Benutzers
sub getuidbyuser {
  my $username = $_[0];
  local (@passwddata, $UIDres, @dummy);
  
  open PASSWD, '</etc/passwd';
  @passwddata = <PASSWD>;
  close PASSWD;
  
  $UIDres = -1;
  foreach(@passwddata) {
    chomp;
    @dummy = split(/:/,$_,-1);
    if (($dummy[0] eq $username) && ($UIDres eq -1)) {
      $UIDres = $dummy[2];
    }
    undef @dummy;
  }
  return $UIDres;
}

# Ermittelt die GID einer Gruppe
sub getgidbygroup {
  my $grpname = $_[0];
  local (@groupdata, $GIDres, @dummy);
  
  open GROUP, '</etc/group';
  @groupdata = <GROUP>;
  close GROUP;
  
  $GIDres = -1;
  foreach(@groupdata) {
    chomp;
    @dummy = split(/:/,$_,-1);
    if (($dummy[0] eq $grpname) && ($GIDres eq -1)) {
      $GIDres = $dummy[2];
    }
    undef @dummy;
  }
  return $GIDres;
}

# Ermittelt den Benutzer zu einer UID
sub getuserbyuid {
  my $userid = $_[0];
  local (@passwddata, @dummy, $USERres);
  
  open PASSWD, '</etc/passwd';
  @passwddata = <PASSWD>;
  close PASSWD;
  
  $USERres = "";
  foreach(@passwddata) {
    chomp;
    @dummy = split(/:/, $_,-1);
    if (($dummy[2] == $userid) && ($USERres eq "")) {
      $USERres = $dummy[0];
    }
    undef @dummy;
  }
  if ($USERres eq "") {
    $USERres = $userid;
  }
  return $USERres;
}

#Packt alle Gruppen (config.pl) in ein Array
sub getallgroups {
  local (@Gruppen,@groupfile,@grpstr);
  
  open (GROUPF, "</etc/group");
  @groupfile = <GROUPF>;
  close GROUPF;
  foreach (@groupfile) {
    chomp;
    @grpstr = split(/:/,$_,-1);
    if (@grpstr != 0) {
      if (($grpstr[2]>=$FIRST_CH_GID) & ($grpstr[2]<=$LAST_CH_GID)) {
        if ($GRUPPENRECHT{$grpstr[0]}==0) {
          push @Gruppen, $_;
        }
      }
    }
  }
  return @Gruppen;
}

sub getallallgroups {
  local (@Gruppen,@groupfile,@grpstr);
  open (GROUPF, "</etc/group");
  @groupfile = <GROUPF>;
  close GROUPF;
  foreach (@groupfile) {
    chomp;
    @grpstr = split (/:/,$_,-1);
    if (($grpstr[2]>=$FIRST_CH_GID) & ($grpstr[2]<=$LAST_CH_GID)) {
      if ($GRUPPENRECHT{$grpstr[0]}==0) {
        push @Gruppen, $grpstr[0];
      }
    }
  }
  return @Gruppen;
}

#Ermittelt den Gruppenname
sub getgroupbygid {
  my $groupid = $_[0];
  local (@groupdata, @dummy, $GROUPres);
  
  open GROUP, '</etc/group';
  @groupdata = <GROUP>;
  close GROUP;
  
  $GROUPres = "";
  foreach(@groupdata) {
    chomp;
    @dummy = split(/:/, $_,-1);
    if (($dummy[2] == $groupid) && ($GROUPres eq "")) {
      $GROUPres = $dummy[0];
    }
    undef @dummy;
  }
  if ($GROUPres eq "") {
    $GROUPres = $groupid;
  }
  return $GROUPres;
}

# Gibt alle Benutzer in html-Listenform zurueck
sub html_userlist {
  my $grpname = $_[0];
  local($res, @userliste);
  
  $res = "";
  @userliste = getuserlist();
  foreach(@userliste) {
    $res = "$res<OPTION>$_";
  }
  return $res;
}

# Gibt die Gruppen eines Benutzers in HTML-Listenform aus.(+ alle anderen)
sub html_groups {
  my $username = $_[0];
  local(@gruppenliste, @alleGruppen, $res, @outGruppen, $equal, $tmp);
  
  @gruppenliste = getgroups($username);
  @alleGruppen = getallallgroups();
  shift @gruppenliste;
  $equal = 0;
  foreach(@gruppenliste) {
    $equal = 0;
    foreach $tmp (@alleGruppen) {
      if ($tmp eq $_) {$equal=-1;}
    }
    if ($equal == 0) {push @outGruppen, $_};
  }
  @outGruppen = (@outGruppen, @alleGruppen);
  @outGruppen = sort @outGruppen;
  $res = "";
  foreach(@outGruppen) {
    if ($GRUPPENRECHT{$_}==0) {
      $equal = 0;
      foreach $tmp (@gruppenliste) {
        if ($_ eq $tmp) {$equal = -1; }
      }
      if ($equal == -1) {
        $res = sprintf "%s<OPTION SELECTED>%s",$res,$_;
      } else {
        $res = sprintf "%s<OPTION>%s",$res,$_;
      }
    }
  }
  return $res;
}

# Gibt die Shells in /etc/shells in HTML-Drop-Down Style aus. Der Parameter
# gibt die aktuelle Shell an.
sub html_shells {
  my $shellname = $_[0];
  local (@shelllist, $html_str);
  
  open SHELLS, '</etc/shells';
  @shelllist = <SHELLS>;
  close SHELLS;
  if ($shellname eq "") {
    $html_str = "<OPTION VALUE=\"\" SELECTED>Keine Shell";
  } else {
    $html_str = "<OPTION VALUE=\"\">Keine Shell";
  }
  foreach (@shelllist) {
    chomp;
    if (($_ ne "")&&(substr($_,0,1) ne "#")) {
      if ($shellname eq $_) {
        $html_str = sprintf "%s<OPTION SELECTED>%s", $html_str, $_; 
      } else {
        $html_str = sprintf "%s<OPTION>%s", $html_str, $_;
      }
    }
  }
  return $html_str;
}

# Diese Routine gibt eine html-liste der SA-Rechte aus. der Parameter gibt
# Das aktuelle Recht an. (Zahl von 0-3)
sub html_saright {
  my $sarecht = $_[0];
  my $userright = $_[1];
  local ($html_str);
  
  if (($sarecht<1)&&($sarecht>3)) {
    $html_str = "<OPTION VALUE=\"__nosaright__\" SELECTED>Keine";
  } else {
    $html_str = "<OPTION VALUE=\"__nosaright__\">Keine";
  }
  if ($sarecht==3) {
    $html_str = sprintf "%s<OPTION SELECTED>$SAGRUPPENNAMEN[2]", $html_str;
  } else {
    $html_str = sprintf "%s<OPTION>$SAGRUPPENNAMEN[2]", $html_str;
  }
  if ($userright == 1) {
    if ($sarecht==2) {
      $html_str = sprintf "%s<OPTION SELECTED>$SAGRUPPENNAMEN[1]", $html_str;
    } else {
      $html_str = sprintf "%s<OPTION>$SAGRUPPENNAMEN[1]", $html_str;
    }
    if ($sarecht==1) {
      $html_str = sprintf "%s<OPTION SELECTED>$SAGRUPPENNAMEN[0]", $html_str;
    } else {
      $html_str = sprintf "%s<OPTION>$SAGRUPPENNAMEN[0]", $html_str;
    }
  }
  return $html_str;
}

# Gibt einen HTML-String zurueck. Der Parameter ist eine GID, der
# die ausgewaehlte Gruppe angibt.
sub html_allgroups {
  my $grpname = $_[0];
  local($htmlstr, @grps, @tmpstr);
  
  @grps = getallgroups();
  $htmlstr = "";
  foreach(@grps) {
    @tmpstr = split(/:/,$_,-1);
    if ($tmpstr[2] eq $grpname) {
      $htmlstr = sprintf "%s<OPTION SELECTED>%s",$htmlstr,$tmpstr[0];
    } else {
      $htmlstr = sprintf "%s<OPTION>%s",$htmlstr,$tmpstr[0];
    }
  }
  return $htmlstr;
}

# Wandelt Sonderzeichen in Standart zeichen.
sub convert_special_chars {
  my $resstr2 = $_[0];
  local ($resstr);
  
  $resstr = $resstr2;
  $resstr =~ s//ae/g;
  $resstr =~ s//Ae/g;
  $resstr =~ s//oe/g;
  $resstr =~ s//Oe/g;
  $resstr =~ s//ue/g;
  $resstr =~ s//Us/g;
  $resstr =~ s//ss/g;
  return $resstr;
}

# Erstellt einen Account.
# Parameter: Liste mit 8 Feldern
# _[0] = VollerName _[1] = username, _[2] = passwort, _[3] = homedir
# _[4] = SA-Recht, _[5] = Klasse, _[6] = Shell _[7] = ExtraStr
# _[1] = "", dann ist username = xx0000 xx=initialen 0000 = Zahl ab 1000
# _[2] = "", dann ist passwort = Vorname in klein
# _[3] = "", dann ist homedir = "/home/[username]"
# _[6] = "", dann ist shell = "Keine Shell"
# _[7] ist fuer Klasse, anlegender Lehrer da.
sub createnewuser {
  my $UD_fullname = $_[0];
  my $UD_username = $_[1];
  my $UD_password = $_[2];
  my $UD_homedir = $_[3];
  my $UD_saright = $_[4];
  my $UD_class = $_[5];
  my $UD_shell = $_[6];
  my $UD_other = $_[7];
  local ($SYSres, @RESvals, $sagroup, %UIDS, %LOGNAMES, $UID);
  local ($VORNAME, $NAME, $ini, $ini1, $ini2, $LOGNAME, $ID, $SARDATA);

  ## HASH-Tables aus /etc/passwd erzeugen
  setpwent;
  while (@passwd=getpwent) {
        $UIDS{$passwd[2]} = $passwd [0];
        $LOGNAMES{$passwd[0]} = $passwd [2];
  }
  endpwent;

  # Unsichere Zeichen entfernen ...
  $UD_fullname = convert_special_chars($UD_fullname);
  $UD_fullname =~ s/[^a-zA-Z0-9\-\ \_]//g;
  $UD_username =~ s/[^a-zA-Z0-9\-\_]//g;
  $UD_password =~ s/[^a-zA-Z0-9\-\ \_]//g;
  $UD_shell =~ s/[^a-zA-Z0-9\-\_\/]//g;
  $UD_homedir =~ s/[^a-zA-Z0-9\-\_\/]//g;
  $UID = $FIRST_NEW_UID;
  #naechste UID finden
  while (exists $UIDS{$UID}) {
        $UID++;
  }
  if ($UD_fullname eq "") {
    print "Benutzer ohne vollen Namen nicht hinzugefgt.<br>\n";
  } else {
    $VORNAME = (split /\ /, $UD_fullname)[0];
    $VORNAME =~ s/[^a-zA-Z0-9\-\_]//g;
    $NAME = substr $UD_fullname, length($VORNAME)+1;
    $NAME =~ s/[^a-zA-Z0-9\-\_]//g;
    if ($UD_password eq "") {$UD_password = lc($VORNAME); }
    if ($UD_username eq "") {
      $ini1=lc(substr($VORNAME, 0, 1));
      $ini2=lc(substr($NAME, 0, 1));
      $ini="$ini1$ini2";
      $ini =~ tr//ooaauus/;
  
      $ID=1001;
      $LOGNAME="$ini$ID"; 
      while (exists $LOGNAMES{$LOGNAME})
      {
         $ID++; 
         $LOGNAME="$ini$ID"; 
      }
      $UD_username = $LOGNAME;
    }
    if ($UD_homedir eq "") {
      $UD_homedir = "/home/$UD_username";
    }
    $SARDATA = "";
    if ($UD_saright == 1) {$SARDATA = $SAGRUPPENNAMEN[0];}
    if ($UD_saright == 2) {$SARDATA = $SAGRUPPENNAMEN[1];}
    if ($UD_saright == 3) {$SARDATA = $SAGRUPPENNAMEN[2];}
    @args=("/usr/sbin/useradd","-u","$UID","-s","$UD_shell");
    @args=(@args,"-c","$VORNAME $NAME","-m","-g","users");
    if ($SARDATA ne "") {@args=(@args,"-G","$SARDATA");}
    @args=(@args,"$UD_username");
    $SYSres = system(@args);
    if ($SYSres==0) {
      print "Benutzer <TT>$UD_username</TT> ($UD_fullname, Passwort:<TT>$UD_password</TT>) hinzugef&uuml;gt<BR>\n";
      @args=("/usr/bin/chfn","-o",$UD_other,$UD_username);
      system(@args);
      open (PASSCH,"| /usr/sbin/chpasswd");
      print PASSCH "$UD_username:$UD_password\n";
      close(PASSCH);
    } else {
      print "Benutzer <TT>$UD_username</TT> ($UD_fullname) konnte nicht hinzugef&uuml;gt werden.<BR>\n";
    }
  }
}

# Ermittelt eine ID fuer eine Klasse
sub getclassid {
  my $klasse = $_[0];
  local (@passwddata, @passwdentr, @spstrentr, $result, %usedCIDs);
  
  open (PASSWD, "</etc/passwd");
  @passwddata = <PASSWD>;
  close (PASSWD);
  $resultat = "";
  foreach(@passwddata) {
    @passwdentr = split(/:/,$_,-1);
    @spstrentr = split(/\,/,$passwdentr[4],-1);
    undef(@passwdentr);
    @passwdentr = split(/\;/,$spstrentr[4],-1);
    $usedCIDs{$passwdentr[1]}=1;
    if ($passwdentr[0] eq $klasse) {
      $result = $passwdentr[1];
    }
    undef(@passwdentr);
    undef(@spstrentr);
  }
  if ($result eq "") {
    $result = 1000;
    while ($usedCIDs{$result}==1) {
      $result++;
    }
  }
  return $result;
}

# Ermittelt den Name eines Benutzers
sub getlname {
  my $username = $_[0];
  local (@userdata);
  
  @userdata = getuserdata($username);
  return $userdata[2];
}

# Ermittelt die Benutzer, die in einer Gruppe sind.
sub getuseringroup {
  my $grpname = $_[0];
  local (@grpdata, $res, @dummy, @userliste);
  
  $res="";
  open(GROUP,"</etc/group");
  @grpdata = <GROUP>;
  close(GROUP);
  foreach(@grpdata) {
    chomp;
    @dummy = split(/:/,$_,-1);
    if ($dummy[0] eq $grpname) {
      @userliste = split(/\,/,$dummy[3]);
      foreach(@userliste) {
        $res = "$res<A HREF=\"../useradmin/shuserdata.cgi?username=$_\">$_";
      }
    }
    undef(@dummy);
  }
  return $res;
}

# Erstellt eine neue Gruppe
# Paramter [0]=grpname
sub createnewgroup {
  my $grpname = $_[0];
  local(@args,$SYSres);
  
  @args=("/usr/sbin/groupadd",$grpname);
  $SYSres = system(@args);
  if ($SYSres) {
    print "Die Gruppe <TT>$grpname</TT> konnte nicht angelegt werden<BR>";
  } else {
    print "Die Gruppe <TT>$grpname</TT> wurde angelegt<BR>";
  }
}

sub getlehrer {
  my $username = $_[0];
  local(@userdata);
  
  @userdata = getuserdata($username);
  
  return $userdata[11];
}

sub getauth {
  my $saright = $_[0];
  local(@moddata);
  
  open (MODINFO, "<./modinfo.dat");
  @moddata = <MODINFO>;
  close(MODINFO);
  if (($saright == 1)&&($moddata[3]==1)) {return 1;}
  if (($saright == 2)&&($moddata[4]==1)) {return 1;}
  if (($saright == 3)&&($moddata[5]==1)) {return 1;}
  return '';
}

1; # Diese Zeile ist Ntig !!!
